; VMM Basic Example by Balrog Software
; OpenGL Cube example ported from PureBasic example

Declare hdc.l, WindowX.l, WindowY.l, WindowWidth.l, WindowHeight.l, hWnd.l, Quit.b
Declare Event.l, hMem.l, pixformat.l, hrc.l, Flags.l, ZoomFactor.f, FlatMode.l
Declare RotateSpeedX.f, RotateSpeedY.f, RotateSpeedZ.f
Declare RollAxisX.f, RollAxisY.f, RollAxisZ.f

; Distance of the camera. Negative value = zoom back

ZoomFactor=1.5

; OpenGL Constants

#GL_DEPTH_BUFFER_BIT =   256
#GL_COLOR_BUFFER_BIT = 16384
#GL_MODELVIEW        =  5888
#GL_TRIANGLES        =     4
#GL_LIGHTING         =  2896
#GL_QUADS            =     7
#GL_LIGHT0           = 16384
#GL_PROJECTION       =  5889
#GL_FLAT             =  7424
#GL_SMOOTH           =  7425
#GL_DEPTH_TEST       =  2929
#GL_CULL_FACE        =  2884

#PIXEL_FORMAT_FLAGS  =    37

; Procedure to set Pixel format descriptor Structure

Procedure SetPFD()
  hMem=AllocateMemory(1, 40)
  If hMem!=0
    PokeW(UseMemory(1), 40)
    PokeW(UseMemory(1) + 2, 1)
    PokeL(UseMemory(1) + 4, #PIXEL_FORMAT_FLAGS)
    PokeB(UseMemory(1) + 9, 16)
    PokeB(UseMemory(1) + 23, 16)
  EndIf
EndProcedure(hMem)

Procedure DrawCube(procdc.l)
  
  ; Save the original Matrix coordinates
  
  glPushMatrix()
  glMatrixMode(#GL_MODELVIEW)
  
  ;  move it forward a bit
  
  glTranslatef(0.0, 0.0, ZoomFactor) 
  
  ; rotate around X, Y and Z axis
  
  glRotatef(RollAxisX, 1.0, 0.0, 0.0)
  glRotatef(RollAxisY, 0.0, 1.0, 0.0)
  glRotatef(RollAxisZ, 0.0, 0.0, 1.0)
  
  Add(RollAxisX, RotateSpeedX)
  Add(RollAxisY, RotateSpeedY)
  Add(RollAxisZ, RotateSpeedZ)
  
  ; clear framebuffer and depth-buffer
  
  glClear(FnOr(#GL_COLOR_BUFFER_BIT, #GL_DEPTH_BUFFER_BIT))
  
  ; draw the faces of a cube
  ; draw colored faces
  
  glDisable(#GL_LIGHTING)
  glBegin(#GL_QUADS)
  
  ; Build a face, composed of 4 vertex ! 
  ; glBegin() specify how the vertexes are considered. Here a group of
  ; 4 vertexes (GL_QUADS) form a rectangular surface.
  
  ; Now, the color stuff: It's r,v,b but with float values which
  ; can go from 0.0 To 1.0 (0 is .. zero And 1.0 is full intensity) 
  
  glNormal3f( 0.0, 0.0, 1.0)
  glColor3f ( 0.0, 0.0, 1.0)
  glVertex3f( 0.5, 0.5, 0.5)
  glColor3f ( 0.0, 1.0, 1.0)
  glVertex3f(-0.5, 0.5, 0.5)
  glColor3f ( 1.0, 1.0, 1.0)
  glVertex3f(-0.5,-0.5, 0.5)
  glColor3f ( 0.0, 0.0, 0.0)
  glVertex3f( 0.5,-0.5, 0.5) 
  
  ; The other face is the same than the previous one 
  ; except the colour which is nice blue To white gradiant
  
  glNormal3f( 0.0, 0.0,-1.0)
  glColor3f ( 0.0, 0.0, 1.0)
  glVertex3f(-0.5,-0.5,-0.5)
  glColor3f ( 0.0, 0.0, 1.0)
  glVertex3f(-0.5, 0.5,-0.5)
  glColor3f ( 1.0, 1.0, 1.0)
  glVertex3f( 0.5, 0.5,-0.5)
  glColor3f ( 1.0, 1.0, 1.0)
  glVertex3f( 0.5,-0.5,-0.5)
  
  glEnd()
  
  ; draw shaded faces
  
  glEnable(#GL_LIGHTING)
  glEnable(#GL_LIGHT0)
  glBegin(#GL_QUADS)
  
  glNormal3f( 0.0, 1.0, 0.0)
  glVertex3f( 0.5, 0.5, 0.5)
  glVertex3f( 0.5, 0.5,-0.5)
  glVertex3f(-0.5, 0.5,-0.5)
  glVertex3f(-0.5, 0.5, 0.5)
  
  glNormal3f( 0.0,-1.0, 0.0)
  glVertex3f(-0.5,-0.5,-0.5)
  glVertex3f( 0.5,-0.5,-0.5)
  glVertex3f( 0.5,-0.5, 0.5)
  glVertex3f(-0.5,-0.5, 0.5)
  
  glNormal3f( 1.0, 0.0, 0.0)
  glVertex3f( 0.5, 0.5, 0.5)
  glVertex3f( 0.5,-0.5, 0.5)
  glVertex3f( 0.5,-0.5,-0.5)
  glVertex3f( 0.5, 0.5,-0.5)
  
  glNormal3f(-1.0, 0.0, 0.0)
  glVertex3f(-0.5,-0.5,-0.5)
  glVertex3f(-0.5,-0.5, 0.5)
  glVertex3f(-0.5, 0.5, 0.5)
  glVertex3f(-0.5, 0.5,-0.5)
  
  glEnd()
  
  glPopMatrix()
  glFinish()
  
  SwapBuffers(procdc)
  
EndProcedure()

Procedure HandleError(Result.l, Text.l)
  If Result=0
    MessageBox(0, Text, "OpenGL Error", 0)
    EndProgram
  EndIf
EndProcedure()

; Enable or disable the 'Flat' rendering

FlatMode=0

; The window & GLViewport dimensions

WindowWidth = 320 + (GetSystemMetrics(#SM_CXBORDER) * 2)
WindowHeight = 240 + (GetSystemMetrics(#SM_CYBORDER) * 2) + GetSystemMetrics(#SM_CYCAPTION)

; The speed of the rotation for the 3 axis

RotateSpeedX=5.0
RotateSpeedY=0.0
RotateSpeedZ=5.0

; Set window position centered on the screen

WindowX=(GetSystemMetrics(#SM_CXSCREEN) / 2) - (WindowWidth / 2)
WindowY=(GetSystemMetrics(#SM_CYSCREEN) / 2) - (WindowHeight / 2)

hWnd=WindowCreate("VMM Basic and OpenGL", WindowX, WindowY, WindowWidth, WindowHeight, FnOr(FnOr(#WS_VISIBLE, #WS_BORDER), #WS_SYSMENU))

If hWnd!=0 And SetPFD()!=0
  
  hdc=GetDC(hWnd)
  
  pixformat=ChoosePixelFormat(hdc, hMem)
  
  HandleError(SetPixelFormat(hdc, pixformat, hMem), "SetPixelFormat()")
  
  hrc=wglCreateContext(hdc)
  
  HandleError(wglMakeCurrent(hdc, hrc), "wglMakeCurrent()")
  
  glMatrixMode(#GL_PROJECTION)
  
  ; position viewer 
  
  glMatrixMode(#GL_MODELVIEW)
  glTranslatef(0.0, 0.0, -2.0)
  
  If FlatMode=1
    glShadeModel(#GL_FLAT) 
  Else
    glShadeModel(#GL_SMOOTH) 
  EndIf
  
  ; Enabled, it slowdown a lot the rendering. It's to be sure than the
  ; rendered objects are inside the z-buffer.
  
  glEnable(#GL_DEPTH_TEST)
  
  ; This will enhance the rendering speed as all the back face will be
  ; ignored. This works only with CLOSED objects like a cube... Singles
  ; planes surfaces will be visibles only on one side. 
  
  glEnable(#GL_CULL_FACE)
  
  ;  A rectangle which define the OpenGL output zone 
  
  HandleError(glViewport (0, 0, WindowWidth - 30, WindowHeight - 30), "GLViewPort()") 
  
  While Quit=0
    
    ProcessEvent()
    
    Event=MessageID()
    
    If Event=#WM_CLOSE 
      Quit=1
    EndIf
    
    DrawCube(hdc)
    Sleep(1)
    
  Wend
  
  If hMem!=0 : FreeMemory(1) : EndIf
  
EndIf

EndProgram